#
# See the file "LICENSE" for the full license governing this code.
#
# frozen_string_literal: true

require 'win32/registry' if Facter.value(:kernel) == 'windows'

newproperty(:autostart) do
  include EasyType
  include Puppet_X::EnterpriseModules::Oracle::Access
  include Puppet_X::EnterpriseModules::Oracle::Settings

  desc <<-DESC
    Add autostart to the oratab entry.
  DESC

  newvalues(:true, :false)

  defaultto(:true)
  data_type('Optional[Boolean]')
  # rubocop:disable all
  to_translate_to_resource do |raw_resource|
    sid = raw_resource['SID']
    if Puppet::Util::Platform.windows?
      oracle_home = configuration_value_for(sid, 'oracle_home')
      ora_install_homes = Facter.value('ora_install_homes') || {'product_version' => {}}
      unless ora_install_homes['product_version'].empty?
        oracle_home_name = ora_install_homes['product_version'][oracle_home]['name']
        regkey = Win32::Registry::HKEY_LOCAL_MACHINE.open("SOFTWARE\\Oracle\\KEY_#{oracle_home_name}")
        autostart = regkey["ORA_#{sid}_AUTOSTART"]
        if !autostart.to_s.empty?
          autostart == 'TRUE' ? :true : :false
        else
          nil
        end
      else
        nil
      end
    else
      oratab = Puppet_X::EnterpriseModules::Oracle::OraTab.new
      if oratab.entry_exists?(sid)
        oratab.entry_autostart?(sid).upcase == 'Y' ? :true : :false
      else
        nil
      end
    end
  end
  # rubocop:enable all

  on_modify do
    case should.to_s
    when 'true'
      old_autostart = 'N'
      autostart = 'Y'
    when 'false'
      old_autostart = 'Y'
      autostart = 'N'
    end
    if Puppet::Util::Platform.windows?
      if autostart == 'Y'
        Puppet::Util::Execution.execute("#{oracle_home(instance)}\\bin\\oradim.exe -edit -sid #{instance} -startmode auto", { :failonfail => true, :combine => true })
      else
        Puppet::Util::Execution.execute("#{oracle_home(instance)}\\bin\\oradim.exe -edit -sid #{instance} -startmode manual", { :failonfail => true, :combine => true })
      end
    else
      oratab = Puppet_X::EnterpriseModules::Oracle::OraTab.new
      if oratab.entry_exists?(instance)
        oratab.remove_entry(instance, oracle_home(instance), old_autostart)
        oratab.ensure_entry(instance, oracle_home(instance), autostart)
      end
    end
  end

  def insync?(is)
    return true unless resource.contained_by.nil?

    is.to_s == should.to_s
  end

  def oracle_home(db)
    configuration_value_for(db, 'oracle_home')
  end
end

def autostart
  self[:autostart].to_s == 'true' ? 'Y' : 'N'
end
