# frozen_string_literal: true

#
# See the file "LICENSE" for the full license governing this code.
#
newproperty(:preferred_instances, :array_matching => :all) do
  include Puppet_X::EnterpriseModules::Oracle::ServiceProperty
  include EasyType::ArrayProperty
  include Puppet_X::EnterpriseModules::Oracle::Commands

  desc <<-DESC
    A list of preferred instances on which the service runs when the database is administrator managed.

    Here is an example on how to use this:

        ora_service{'my_service':
          ...
          prefered_instances => ['SID1', 'SID2'],
          ...
        }

    When an instance is specified that does not or not yes exist in the cluster, Puppet will
    ignore it.

    This is a cluster only property. On single node database this property will be ignored. If you use it,
    Puppet will issue a warning.

  DESC

  column_name 'Preferred instances'
  data_type('Optional[Variant[String[1],Array]]')

  defaultto do
    cluster_instances
  rescue RuntimeError
    # When the database doesn't exist yet, use a nil. Later we will fix the
    # default value back again.
    []
  end

  def insync?(is)
    is = [] if is.to_s == 'absent'
    if cluster?
      is.sort == (should - unknown_instances).sort
    else
      Puppet.info "#{path}: #{name} has no effect on non clustered service." unless should == [resource.sid]
      true
    end
  end

  to_translate_to_resource do |raw_resource|
    value = super(raw_resource)
    value.nil? ? nil : value.to_s.split(',').map(&:strip)
  end

  on_create do
    apply('create')
  end

  on_modify do
    # modify is handled in after_modify
    nil
  end

  after_modify do
    if cluster?
      #
      # If it contains unknown instances at this point in time, filter them out
      #
      if resource[:available_instances]
        if unknown_instances?(resource[:available_instances])
          unless @warned
            Puppet.warning "#{path.gsub(/preferred_instances$/,
                                        'available_instances')}: Skipping unknown available_instances #{unknown_instances(resource[:available_instances]).join(',')}"
          end
          @warned = true
          sane_avail_value = resource[:available_instances] - unknown_instances(resource[:available_instances])
          avail = if sane_avail_value.any?
                    provider.send(:update_available_instances, sane_avail_value)
                  else
                    ''
                  end
        else
          avail = provider.send(:update_available_instances, resource[:available_instances])
        end
      end
      if unknown_instances?
        sane_value = value - unknown_instances
        fail "#{resource}: No known instances specified for property preferred_instances" unless sane_value.any?

        Puppet.info "#{resource}: Skipping unknown instances #{unknown_instances.join(',')} continue with #{sane_value.join(',')}" unless @warned
        @warned = true
        pref = provider.send("update_#{self.class.name}", sane_value)
      else
        pref = provider.send("update_#{self.class.name}", value)
      end
      srvctl("modify service -d #{db_for(sid)} -service #{resource[:service_name]} #{pref} #{avail}", :sid => sid_for(sid))
    else
      Puppet.warning "#{resource}: preferred_instances has no effect on non clustered service." unless resource.preferred_instances == [resource.sid]
    end
  end

  private

  def apply(type)
    if cluster?
      #
      # If it contains unknown instances at this point in time, filter them out
      #
      if unknown_instances?
        sane_value = value - unknown_instances
        fail "#{resource}: No known instances specified for property preferred_instances" unless sane_value.any?

        Puppet.info "#{resource}: Skipping unknown instances #{unknown_instances.join(',')} continue with #{sane_value.join(',')}" unless @warned
        @warned = true
        provider.send("#{type}_#{self.class.name}", sane_value)
      elsif value.any?
        provider.send("#{type}_#{self.class.name}", value)
      end
    else
      Puppet.warning "#{resource}: preferred_instances has no effect on non clustered service." unless resource.preferred_instances == [resource.sid]
    end
  end

  #
  # If the value is empty, use all current instances as default
  #
  def requested_value
    value.empty? ? cluster_instances : value
  end

  def unknown_instances?(val = value)
    unknown_instances(val).any?
  end

  def unknown_instances(val = value)
    if val == value
      requested_value - cluster_instances
    else
      val - cluster_instances
    end
  end

  def known_instances
    requested_value & cluster_instances
  end
end
