# frozen_string_literal: true

#
# See the file "LICENSE" for the full license governing this code.
#

$LOAD_PATH.unshift(File.absolute_path("#{__dir__}/../.."))
$LOAD_PATH.unshift(File.absolute_path("#{__dir__}/../../../../easy_type/lib"))
require 'easy_type'
require 'easy_type/array_property'
require 'puppet_x/enterprisemodules/oracle/access'
require 'puppet_x/enterprisemodules/oracle/read_only_property'
require 'puppet_x/enterprisemodules/oracle/password'
require 'puppet_x/enterprisemodules/oracle/create_only'
require 'puppet_x/enterprisemodules/oracle/title_parser'
require 'puppet_x/enterprisemodules/oracle/grant_property'
require 'puppet_x/enterprisemodules/oracle/grant_validator'
require 'puppet_x/enterprisemodules/oracle/utilities'
require 'puppet_x/enterprisemodules/oracle/default_sid'
#
# Create a new type oracle_user. Oracle user, works in conjunction
# with the Resource provider
#
Puppet::Type.newtype(:ora_user) do
  # docs
  class ::Puppet::Type::Ora_user
    include EasyType
    include Puppet_X::EnterpriseModules::Oracle::Access
    include Puppet_X::EnterpriseModules::Oracle::DefaultSid
    include Puppet_X::EnterpriseModules::Oracle::Utilities
    extend Puppet_X::EnterpriseModules::Oracle::TitleParser
    include Puppet_X::EnterpriseModules::Oracle::GrantValidator

    desc <<-DESC
      This type allows you to manage a user inside an Oracle database.

      It recognises most of the options that [CREATE USER](http://docs.oracle.com/cd/B28359_01/server.111/b28286/statements_8003.htm#SQLRF01503) supports. Besides these options, you can also use this type to manage the grants and the quota's for this user.

          ora_user{user_name@sid:
            temporary_tablespace      => temp,
            default_tablespace        => 'my_app_ts',
            password                  => 'verysecret',
            require                   => Ora_tablespace['my_app_ts'],
            grants                    => ['SELECT ANY TABLE', 'CONNECT', 'CREATE TABLE', 'CREATE TRIGGER'],
            quotas                    => {
                                            "my_app_ts"  => 'unlimited'
                                          },
          }
    DESC

    ensurable

    def validate
      return if self[:ensure].to_s == 'absent'

      super
      validate_double_values(:default_roles)
      fail 'Property password and raw_password are mutualy exclusive but both specified.' if self[:password] && self[:raw_password]
    end

    to_get_raw_resources do
      sql_on_all_primary_database_sids %(col oracle_maintained form a18
                                          select * from dba_users)
    end

    on_create do
      if self[:raw_password]
        [:sql, "create user #{username} identified by values '#{raw_password}'", { :skip_without_properties => true, :sid => sid }]
      else
        self[:password] ||= generated_password
        [:sql, "create user #{username} identified by \"#{password}\"", { :skip_without_properties => true, :sid => sid }]
      end
    end

    on_modify do
      Puppet.warning "Changing Oracle maintained user #{username}@#{sid}" if ignore_oracle_maintained_warnings.to_s != 'true' && (provider.oracle_maintained.to_s == 'true')
      [:sql, "alter user #{username}", { :skip_without_properties => true, :sid => sid }]
    end

    on_destroy do
      Puppet.warning "Dropping Oracle maintained user #{username}@#{sid}" if ignore_oracle_maintained_warnings.to_s != 'true' && (provider.oracle_maintained.to_s == 'true')
      [:sql, "drop user #{username} cascade", { :sid => sid }]
    end

    map_title_to_sid(:username)

    parameter :name
    parameter :username
    parameter :sid
    parameter :create_only
    parameter :ignore_oracle_maintained_warnings

    property  :default_tablespace
    property  :expired
    property  :grants
    property  :common_grants
    property  :common_grants_with_admin
    property  :common_revoked
    property  :common_granted
    property  :common_revoked_with_admin
    property  :common_granted_with_admin
    property  :revoked
    property  :granted
    property  :grants_with_admin
    property  :revoked_with_admin
    property  :granted_with_admin
    property  :default_roles # This one needs to be after grants
    property  :locked
    property  :password
    property  :raw_password
    property  :profile
    property  :quotas
    property  :temporary_tablespace
    property  :container
    property  :oracle_maintained

    ora_autorequire(:ora_tablespace, [:default_tablespace, :temporary_tablespace, proc { quotas ? quotas.keys : nil }])
    ora_autorequire(:ora_profile, :profile)
    ora_autorequire(:ora_role, :default_roles)

    def grant_key
      username
    end

    #
    # This method is used to specify the Column table for fetching the granted roles
    #
    class << self
      attr_reader :grantee_column
    end
    @grantee_column = 'USERNAME'
  end
end
