#
# ::fact_caching
#
# @summary This class ensure's that the ora_config facts that can be cached and are cached with the correct settings.
# You can change all TTL settings for the facts if an other setting is more fitting for your setup.
#
# @param [Boolean] enabled
#    When set to true, this enables facter caching for the ora_install module facts.
#    Beware that you need to have Facter version 4.1.0 or higher to enabled this. When you set this value to true, automatically TTL settings for most of the ora_install facts are set to a default value. For most installations, the default settings are fine.
#    When you set this value to `true`, the [facter setting `force-dot-resolution`](https://tickets.puppetlabs.com/browse/PUP-11089) is enabled. You have to ensure that all of your facts keep on working with this setting.
#
# @param [Optional[String[1]]] ora_asm_diskgroups_ttl
#    The Time to Live for the `ora_asm_diskgroups` fact.
#
# @param [Optional[String[1]]] ora_asm_running_ttl
#    The Time to Live for the `ora_asm_running` fact.
#
# @param [Optional[String[1]]] ora_asm_volumes_ttl
#    The Time to Live for the `ora_asm_volumes` fact.
#
# @param [Optional[String[1]]] ora_is_cluster_ttl
#    The Time to Live for the `ora_is_cluster` fact.
#
# @param [Optional[String[1]]] ora_is_container_db_ttl
#    The Time to Live for the `ora_is_container_db` fact.
#
# @param [Optional[String[1]]] ora_is_pluggable_db_ttl
#    The Time to Live for the `ora_is_pluggable_db` fact.
#
# @param [Optional[String[1]]] ora_is_primary_db_ttl
#    The Time to Live for the `ora_is_primary_db` fact.
#
# @param [Optional[String[1]]] ora_is_root_db_ttl
#    The Time to Live for the `ora_is_root_db` fact.
#
# @param [Optional[String[1]]] ora_is_seed_db_ttl
#    The Time to Live for the `ora_is_seed_db` fact.
#
# @param [Optional[String[1]]] ora_version_ttl
#    The Time to Live for the `ora_version` fact.
#
#
# See the file "LICENSE" for the full license governing this code.
#
class ora_config::fact_caching (
  Boolean             $enabled,
  Optional[String[1]] $ora_asm_diskgroups_ttl,
  Optional[String[1]] $ora_asm_running_ttl,
  Optional[String[1]] $ora_asm_volumes_ttl,
  Optional[String[1]] $ora_is_cluster_ttl,
  Optional[String[1]] $ora_is_container_db_ttl,
  Optional[String[1]] $ora_is_pluggable_db_ttl,
  Optional[String[1]] $ora_is_primary_db_ttl,
  Optional[String[1]] $ora_is_root_db_ttl,
  Optional[String[1]] $ora_is_seed_db_ttl,
  Optional[String[1]] $ora_version_ttl
) {
  if $enabled {
    include ora_config::clear_caches
    #
    # If enabled, we need to enable the force-dot-resolution
    # setting for facter.
    #
    unless defined(Fact_setting['force-dot-resolution']) {
      fact_setting { 'force-dot-resolution':
        value => true,
      }
    }

    fact_config { 'ora_is_container_db':
      ttl => $ora_is_container_db_ttl,
    }

    fact_config { 'ora_is_root_db':
      ttl => $ora_is_root_db_ttl,
    }

    fact_config { 'ora_is_seed_db':
      ttl => $ora_is_seed_db_ttl,
    }

    fact_config { 'ora_is_pluggable_db':
      ttl => $ora_is_pluggable_db_ttl,
    }

    fact_config { 'ora_is_cluster':
      ttl => $ora_is_cluster_ttl,
    }

    fact_config { 'ora_version':
      ttl => $ora_version_ttl,
    }

    fact_config { 'ora_asm_diskgroups':
      ttl => $ora_asm_diskgroups_ttl,
    }

    fact_config { 'ora_asm_volumes':
      ttl => $ora_asm_volumes_ttl,
    }

    fact_config { 'ora_is_primary_db':
      ttl => $ora_is_primary_db_ttl,
    }

    fact_config { 'ora_asm_running':
      ttl => $ora_asm_running_ttl,
    }
  } else {
    #
    # On puppet versions older than V6, the required libs
    # for fact_config are not installed. Because it also doesn't
    # serve any purpose, we skip this part on those versions.
    #
    if versioncmp($clientversion, '6.0.0') != -1 {
      #
      # Disable fact caching for all ora_config facts
      #
      $ora_config_facts = [
        'ora_is_container_db',
        'ora_is_root_db',
        'ora_is_seed_db',
        'ora_is_pluggable_db',
        'ora_is_cluster',
        'ora_version',
        'ora_asm_diskgroups',
        'ora_asm_volumes',
        'ora_is_primary_db',
        'ora_asm_running',
      ]

      fact_config { $ora_config_facts:
        ttl => absent,
      }
    }
} }
